#!/usr/bin/env python

import unittest
import wordnik
import urllib2

def fake_do_http(uri, headers, body=None, method="GET", beta=False):
    # The real _do_http returns a string run through json.loads or ElementTree.XML.
    return {"fakeKey": "fakeValue", "token": "deadbeef"}

def fake_fail_http(uri, headers, body=None, method="GET", beta=False):
    return None

class TestWordnikModule(unittest.TestCase):
    def setUp(self):
        self.key    = 'deadbeef'
        self.w      = wordnik.Wordnik(api_key=self.key)
        

    def test_bad_init(self):
        self.assertRaises(wordnik.NoAPIKey,
                          wordnik.Wordnik)

    def test_internals(self):
        self.assertEqual(wordnik.DEFAULT_FORMAT, 'json')
        self.assertEqual(wordnik.DEFAULT_URL, 'http://api.wordnik.com/v4')
        self.assertEqual(self.w._api_key, self.key)

    def test_methods(self):
        self.assertEqual(type(self.w.word_get), type(self.w.__init__))

class TestWordnikAPIMethods(unittest.TestCase):
    def setUp(self):
        self.key        = 'deadbeef'
        self.w          = wordnik.Wordnik(api_key=self.key)
        self.w._do_http = fake_do_http
        self.response   = fake_do_http(None, None)

    def test_account_methods(self):
        self.assertEqual(self.w.account_get_api_token_status(),self.response)

        self.assertRaises(wordnik.MissingParameters,self.w.account_get_authenticate)
        self.assertRaises(wordnik.MissingParameters,self.w.account_get_authenticate, "user")
        self.assertEqual(self.w.account_get_authenticate("testuser", password="testpass"),self.response)
                
        self.assertRaises(wordnik.MissingParameters,self.w.account_get_user)
        self.assertEqual(self.w.account_get_user(auth_token="eee"),self.response)

        self.assertRaises(wordnik.MissingParameters,self.w.account_post_authenticate)
        self.assertRaises(wordnik.MissingParameters,self.w.account_post_authenticate, "testuser")
        self.assertEqual(self.w.account_post_authenticate("testuser", body={}),self.response)

    def test_authenticate(self):
        self.assertRaises(TypeError, self.w.authenticate)
        self.assertEqual(self.w.authenticate("user", "pass"), True)
        
    def test_multi(self):
        ## XXX This needs to be done; multi() might need to get rewritten
        pass

    # def test_user_methods(self):
    # 
    #             
    #     self.assertRaises(wordnik.MissingParameters,self.w.user_get_word_of_the_day_list)
    #     self.assertEqual(self.w.user_get_word_of_the_day_list("user"),self.response)
    # 
    #     self.assertRaises(wordnik.MissingParameters,self.w.user_post_word_of_the_day_list)
    #     self.assertRaises(wordnik.MissingParameters,self.w.user_post_word_of_the_day_list, "user")
    #     self.assertEqual(self.w.user_post_word_of_the_day_list("user", "link", body={}),self.response)
    # 
    #     self.assertRaises(wordnik.MissingParameters,self.w.user_put_word_of_the_day_list)
    #     self.assertRaises(wordnik.MissingParameters,self.w.user_put_word_of_the_day_list, "user")
    #     self.assertEqual(self.w.user_put_word_of_the_day_list("user", "link", body={}),self.response)
    # 
    #     self.assertRaises(wordnik.MissingParameters,self.w.user_put_word_of_the_day_list_add)
    #     self.assertRaises(wordnik.MissingParameters,self.w.user_put_word_of_the_day_list_add, "user")
    #     self.assertEqual(self.w.user_put_word_of_the_day_list_add("user", "link", body={}),self.response)

    def test_word_methods(self):
        
        self.assertRaises(wordnik.MissingParameters,self.w.word_get)
        self.assertEqual(self.w.word_get("cat"),self.response)
        
        self.assertRaises(wordnik.MissingParameters,self.w.word_get_audio)
        self.assertEqual(self.w.word_get_audio("cat"),self.response)
          
        self.assertRaises(wordnik.MissingParameters,self.w.word_get_definitions)
        self.assertEqual(self.w.word_get_definitions("cat"),self.response)
        
        self.assertRaises(wordnik.MissingParameters,self.w.word_get_examples)
        self.assertEqual(self.w.word_get_examples("cat"),self.response)
        
        self.assertRaises(wordnik.MissingParameters,self.w.word_get_frequency)
        self.assertEqual(self.w.word_get_frequency("cat"),self.response)
        
        self.assertRaises(wordnik.MissingParameters,self.w.word_get_hyphenation)
        self.assertEqual(self.w.word_get_hyphenation("cat"),self.response)
        
        self.assertRaises(wordnik.MissingParameters,self.w.word_get_phrases)
        self.assertEqual(self.w.word_get_phrases("cat"),self.response)
        
        self.assertRaises(wordnik.MissingParameters,self.w.word_get_pronunciations)
        self.assertEqual(self.w.word_get_pronunciations("cat"),self.response)
        
        self.assertRaises(wordnik.MissingParameters,self.w.word_get_related)
        self.assertEqual(self.w.word_get_related("cat"),self.response)
        
        self.assertRaises(wordnik.MissingParameters,self.w.word_get_top_example)
        self.assertEqual(self.w.word_get_top_example("cat"),self.response)
        
    
    def test_word_list_methods(self):

        self.assertRaises(wordnik.MissingParameters,self.w.word_list_delete)
        self.assertEqual(self.w.word_list_delete("808"),self.response)
        
        self.assertRaises(wordnik.MissingParameters,self.w.word_list_get)
        self.assertEqual(self.w.word_list_get("808"),self.response)
        
        self.assertRaises(wordnik.MissingParameters,self.w.word_list_get_words)
        self.assertEqual(self.w.word_list_get_words("808"),self.response)
        
        self.assertRaises(wordnik.MissingParameters,self.w.word_list_post_delete_words)
        self.assertRaises(wordnik.MissingParameters,self.w.word_list_post_delete_words, "808")
        self.assertEqual(self.w.word_list_post_delete_words("808", body={}),self.response)
        
        self.assertRaises(wordnik.MissingParameters,self.w.word_list_post_words)
        self.assertRaises(wordnik.MissingParameters,self.w.word_list_post_words, "808")
        self.assertEqual(self.w.word_list_post_words("808", body={}),self.response)
        
    def test_word_lists_methods(self):

        self.assertEqual(self.w.word_lists_get(),self.response)

        self.assertRaises(wordnik.MissingParameters,self.w.word_lists_post)
        self.assertEqual(self.w.word_lists_post(body={}),self.response)

    def test_words_methods(self):

        self.assertEqual(self.w.words_get_random_word(),self.response)

        self.assertEqual(self.w.words_get_random_words(),self.response)

        self.assertRaises(wordnik.MissingParameters,self.w.words_get_search)
        self.assertEqual(self.w.words_get_search(query="cat"),self.response)


if __name__ == "__main__":
    unittest.main()
  




# Exceptions
from wordnik import RestfulError, InvalidRelationType, NoAPIKey, MissingParameters

# Globals
from wordnik import DEFAULT_URL, DEFAULT_FORMAT

from wordnik import Wordnik
Wordnik._populate_methods()




import json, re, urllib, urllib2, wordnik
import wordnik

def generate_docs(params, response, summary, path):
    """This will generate the documentation for a function given some information
    about the params, the response (not currently used), the summary, and the path."""
    docstring   = "{0}\n".format(summary)
    docstring  += "{0}\n".format(path)
    
    path_params  = get_path_params(params)
    other_params = get_other_params(params)
   
    if path_params:
        docstring += "\nPath Parameters:\n"
        for param in path_params:
            param_doc  = "  {0}\n".format(param.get('name'))
            docstring += param_doc
 
    if other_params:
        docstring += "\nOther Parameters:\n"
        for param in other_params:
            param_doc  = "  {0}\n".format(param.get('name'))
            docstring += param_doc

    return docstring

def get_path_params(params):
    path_params = []
    for param in [ p for p in params if p['paramType'] == 'path' ]:
        p = {}
        p['name']        = param.get('name') or 'body'
        p['description'] = param.get('description')
        p['paramType']   = param.get('paramType')
        p['required']    = param.get('required')
        path_params.append(p)
    return path_params

def get_query_params(params):
    query_params = []
    for param in [ p for p in params if p['paramType'] == 'query' ]:
        p = {}
        p['name']        = param.get('name') or 'body'
        p['description'] = param.get('description')
        p['paramType']   = param.get('paramType')
        p['required']    = param.get('required')
        query_params.append(p)
    return query_params

def get_other_params(params):
    other_params = []
    for param in [ p for p in params if p['paramType'] != 'path' ]:
        p = {}
        p['name']        = param.get('name') or 'body'
        p['description'] = param.get('description')
        p['paramType']   = param.get('paramType')
        p['required']    = param.get('required')
        other_params.append(p)
    return other_params

def get_required_params(params):
    required_params = []
    for param in [ p for p in params if p['paramType'] != 'path' and p['required'] == True ]:
        p = {}
        p['name']        = param.get('name') or 'body'
        p['description'] = param.get('description')
        p['paramType']   = param.get('paramType')
        p['required']    = param.get('required')
        required_params.append(p['name'])
    return required_params
      
def create_method(name, doc, params, path, httpmethod):
    """The magic behind the dynamically generated methods in the Wordnik object"""
    def _method(self, *args, **kwargs):
        return self._run_command(name, *args, **kwargs)
    
    _method.__doc__  = str(doc)
    _method.__name__ = str(name)
    _method._path    = path
    _method._params  = params
    _method._http    = httpmethod
    
    return _method

def process_args(path, params, args, kwargs):
    """This does all the path substitution and the population of the
    headers and/or body, based on positional and keyword arguments.
    """
    required_params = get_required_params(params)
    given_params    = kwargs.keys()
    query_params    = get_query_params(params)
    
    if not set(given_params).issuperset(set(required_params)):
        notsupplied = set(given_params).symmetric_difference(set(required_params)).intersection(set(required_params))
        raise wordnik.MissingParameters("Some required parameters are missing: {0}".format(notsupplied))
    
    ## get "{format} of of the way first"
    format = kwargs.get('format') or wordnik.DEFAULT_FORMAT
    path   = path.replace('{format}', format) + "?"    
    
    positional_args_re  = re.compile('{([\w]+)}')
    headers             = {}
    body                = None
    
    ## substiture the positional arguments, left-to-right
    for arg in args:
        path = positional_args_re.sub(urllib.quote(arg), path, count=1)

    ## now look through the keyword args and do path substitution
    for arg,value in kwargs.items():
        if arg not in path:
            continue
        bracketedString = "{" + arg + "}"
        pathPattern = re.compile(bracketedString)
        path = pathPattern.sub(urllib.quote(value), path)
        ## we want to remove this item from kwargs (we already used it!)
        kwargs.pop(arg)

    ## if we need to set the HTTP body, we do it in kwargs
    if 'body' in kwargs:
        body = json.dumps(kwargs.pop('body'))
    
    ## handle additional query args
    for param in query_params:
        name = param.get('name')
        if name in kwargs:
            path += "{0}={1}&".format(name, urllib.quote(kwargs.pop(name).__str__()))

    
    ## put all remaining kwargs in the headers
    for arg in kwargs:    
        headers[arg] = urllib.quote(kwargs[arg])

    ## If we still have any unsubstituted params in the path, we need to 
    ## raise an exception.
    
    if positional_args_re.search(path):
        raise wordnik.MissingParameters("Some required parameters are missing: {0}".format(path))
    
    ## similarly, raise and exception if we're missing a keyword arg.
    for param in params:
        if param['paramType'] == 'body' and body == None:
            raise wordnik.MissingParameters("Some required parameters are missing: {0}".format(param))
    
    ## return a 3-tuple of (<URI path>, <headers>, <body>)
    return (path, headers, body)


def uncamel(string):
    """unCamels cameledStrings"""
    s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', string)
    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()

def remove_params(path):
    """Gets rid of all {params} in a URL"""
    param_re = re.compile('{[\w]+}')
    return param_re.subn('', path)[0]
    
def componentize(path):
    """Splits a path on underscores and returns the components"""
    return re.split('[_]+', path)
    
def normalize(path, method):
    """Makes a crazy path + HTTP method look like a sane method name"""
    under_re = re.compile('(^[_]+|[_]+$)')
    repeat_under_re = re.compile('[_]+')
    
    p = remove_params(path)
    p = p.replace('/', '_').replace('.', '_').strip('_')
    components = componentize(p)
    m = [ components[0], method ]
    m.extend(components[1:])
    return uncamel("_".join(m))



"""Python wrapper for the Wordnik API. 

This API implements all the methods described at http://developer.wordnik.com/docs

maintainer: Robin Walsh (robin@wordnik.com)
"""

import helpers
import httplib
try:
    import simplejson as json
except ImportError:
    import json
import os
import urllib
import urllib2
from optparse import OptionParser
from xml.etree import ElementTree
from pprint import pprint
from sys import stdout, stderr

DEFAULT_HOST   = "api.wordnik.com"
DEFAULT_URI    = "/v4"
DEFAULT_URL    = "http://" + DEFAULT_HOST + DEFAULT_URI
FORMAT_JSON = "json"
FORMAT_XML = "xml" 
DEFAULT_FORMAT = FORMAT_JSON

class RestfulError(Exception):
    """Raised when response from REST API indicates an error has occurred."""

class InvalidRelationType(Exception):
    """Raised if Wordnik.related method is passed invalid relation type."""

class NoAPIKey(Exception):
    """Raised if we don't get an API key."""

class MissingParameters(Exception):
    """Raised if we try to call an API method with required parameters missing"""
    
class Wordnik(object):
    
    """
    A generic Wordnik object. Use me to interact with the Wordnik API.
    
    All of my methods can be called in multiple ways. All positional
    arguments passed into one of my methods (with the exception of "format")
    will be substituted for the correponding path parameter, if possible.
    For example, consider the "get_word_examples" method. The URI path is:
    
    /word.{format}/{word}/examples
    
    So we can skip format (default format is JSON) and infer that the first
    positional argument is the word we want examples for. Hence:
    
    Wordnik.word_get_examples('cat')
    
    All other (non-path) arguments are keyword arguments. The "format"
    paramater can be passed in this way as well. Hence:
    
    Wordnik.word_get_examples('cat', format='xml', limit=500)
    
    In the case where you're making a POST, you will need a "body" keyword:
    
    Wordnik.word_list_put(wordListId=1234, body="Some HTTP body")    
    """
    
    
    def __init__(self, api_key=None, username=None, password=None, beta=False):
        """
        Initialize a Wordnik object. You must pass in an API key when
        you make a new Wordnik. We don't validate the API key until the
        first call against the API is made, at which point you'll find
        out if it's good.
        
        If you also pass in a username and password, we will try to get an
        auth token so you can use the Wordnik authenticated methods.
        Alternatively, you can call Wordnik.authenticate(user, pass)
        """
        
        if api_key is None:
            raise NoAPIKey("No API key passed to our constructor")
        
        self._api_key = api_key
        self.username = username
        self.password = password
        self.token    = None
        self.beta     = beta
        
        if username and password:
            try:
                j = json.loads(self.account_get_authenticate(username, password=password))
                self.token = j['token']
            except:
                raise RestfulError("Could not authenticate with the given username and password")
        
        
    @classmethod
    def _populate_methods(klass):
        """This will create all the methods we need to interact with
        the Wordnik API"""
        
        ## there is a directory called "endpoints"
        basedir = os.path.dirname(__file__)
        for filename in os.listdir('{0}/endpoints'.format(basedir)):
            full_path = os.path.join(basedir, 'endpoints', filename)
            if os.path.isfile(full_path):
                j = json.load(open(full_path))
                Wordnik._create_methods(j)
            
    @classmethod
    def _create_methods(klass, jsn):
        """A helper method that will populate this module's namespace
        with methods (parsed directlly from the Wordnik API's output)
        """
        endpoints = jsn['endPoints']
    
        for method in endpoints:
            path = method['path']
            for op in method['operations']:
                summary = op['summary']
                httpmethod = op['httpMethod']
                params = op['parameters']
                response = op['response']
    
                ## a path like: /user.{format}/{username}/wordOfTheDayList/{permalink} (GET)
                ## will get translated into method: user_get_word_of_the_day_list
                methodName  = helpers.normalize(path, httpmethod.lower())
                docs        = helpers.generate_docs(params, response, summary, path)
                method      = helpers.create_method(methodName, docs, params, path, httpmethod.upper())
                
                setattr( Wordnik, methodName, method )
    
    def _run_command(self, command_name, *args, **kwargs):
        if 'api_key' not in kwargs:
            kwargs.update( {"api_key": self._api_key} )
        if self.token:
            kwargs.update( {"auth_token": self.token} )
        
        command                 = getattr(self, command_name)
        (path, headers, body)   = helpers.process_args(command._path, command._params, args, kwargs)
        httpmethod              = command._http
        
        return self._do_http(path, headers, body, httpmethod, beta=self.beta)
        
    def multi(self, calls, **kwargs):
        """Multiple calls, batched. This is a "special case" method
        in that it's not automatically generated from the API documentation.
        That's because, well, it's undocumented. Here's how you use it:
        
        Wordnik.multi( [call1, call2, call3 ], **kwargs)
        
        where each "call" is (word, resource, {param1: value1, ...} )
        So we could form a batch call like so:
        
        calls = [("dog","examples"),("cat","definitions",{"limit":500})]
        
        Wordnik.multi(calls, format="xml")
        
        """
        
        path = "/word.%s?multi=true" % (kwargs.get('format') or DEFAULT_FORMAT)
        
        
        for calls_made, call in enumerate(calls):
            word = call[0]
            resource = call[1]
            if len(call) >= 3:
                otherParams = call[2]
            else:
                otherParams = {}
            ## Add the first resource to the URL
            path += "&resource.{0}={1}/{2}".format(calls_made,word,resource)
            for key,val in otherParams.items():
                ## Add potential extra params to the URL
                path += "&{0}.{1}={2}".format(key, calls_made, val)
        
        headers = { "api_key": self._api_key }
        if self.token:
            headers.update( {"auth_token": self.token} )
        
        return self._do_http(path, headers, beta=self.beta)
    
    def authenticate(self, username, password):
        """A convenience method to get an auth token in case the object was 
        not instantiated with a username and a password.
        """

        try:
            resp = self.account_get_authenticate(username, password=password, 
                                                 format='json')
        except:
            raise RestfulError("Could not authenticate with the given username and password")
        else:
            self.token = resp['token']
            return True
    
    @staticmethod
    def _do_http(uri, headers, body=None, method="GET", beta=False):
        """This wraps the HTTP call. This may get factored out in the future."""
        if body:
            headers.update( {"Content-Type": "application/json"})
        full_uri = DEFAULT_URI + uri
        conn = httplib.HTTPConnection(DEFAULT_HOST)
        if beta:
            conn = httplib.HTTPConnection("beta.wordnik.com")
        conn.request(method, full_uri, body, headers)
        response = conn.getresponse()
        if response.status == httplib.OK:
            text = response.read()
            format_ = headers.get('format', DEFAULT_FORMAT)
            if format_ == FORMAT_JSON:
                return json.loads(text)
            elif format_ == FORMAT_XML:
                return ElementTree.XML(text)
        else:
            print >> stderr, "{0}: {1}".format(response.status, response.reason)
            return None



#!/
#!/usr/bin/env python

import unittest
import wordnik
import urllib2

## please just trust me... D:
params = [{u'paramType': u'path', u'required': False, u'description': u'ID of WordOfTheDayList', u'name': u'date'}, {u'paramType': u'query', u'required': False, u'description': u'Returns future WordOfTheDay items', u'name': u'includeAll'}, {u'name': u'format', u'paramType': u'path', u'required': True, u'allowableValues': u'json,xml', u'defaultValue': u'json', u'description': u'API response format'}]

alt_params = [{u'paramType': u'path', u'required': False, u'description': u'ID of WordOfTheDayList', u'name': u'date'}, {u'paramType': u'query', u'required': True, u'description': u'Returns future WordOfTheDay items', u'name': u'includeAll'} ]

response = [] ## not currently used, so not tested
summary = "Generic function summary"
path = "/fake.{format}/{parameter}/path"

def fake_do_http(uri, headers, body=None, method="GET", beta=False):
    return """{ "fakeKey": "fakeValue" }"""

def fake_fail_http(uri, headers, body=None, method="GET", beta=False):
    return None

class TestHelperFunctions(unittest.TestCase):
    def setUp(self):
        self.h = wordnik.helpers

    def test_generate_docs(self):
        expectedDoc = """Generic function summary
/fake.{format}/{parameter}/path

Path Parameters:
  date
  format

Other Parameters:
  includeAll
"""
        self.assertRaises(TypeError, self.h.generate_docs)
        self.assertRaises(TypeError, self.h.generate_docs, params)
        self.assertRaises(TypeError, self.h.generate_docs, params, response)
        self.assertRaises(TypeError, self.h.generate_docs, params, summary)

        self.assertEqual(self.h.generate_docs(params, response, summary, path), expectedDoc)

    def test_create_method(self):

        doc = self.h.generate_docs(params, response, summary, path)

        self.assertRaises(TypeError, self.h.create_method)
        self.assertRaises(TypeError, self.h.create_method, "name")
        self.assertRaises(TypeError, self.h.create_method, "name", doc)
        self.assertRaises(TypeError, self.h.create_method, "name", doc, params)

        self.assertEqual(type(fake_do_http), type(self.h.create_method("name", doc, params, path, "GET")))
        
    def test_process_args(self):

        self.assertRaises(TypeError, self.h.process_args, path)
        self.assertRaises(TypeError, self.h.process_args, path, params)
        self.assertRaises(TypeError, self.h.process_args, path, params, [])

        expectedPath = """/fake.json/fake%20Param/path?"""
        expectedHeaders = {}
        expectedBody = None

        self.assertEqual(self.h.process_args(path, params, [], { "parameter": "fake Param"} ),
                         (expectedPath, expectedHeaders, expectedBody) )
        
        expectedBody = '{"key": "value"}'        
        self.assertEqual(self.h.process_args(path, params, [], { "parameter": "fake Param", "body": {"key": "value"}} ),
                         (expectedPath, expectedHeaders, expectedBody) )
        expectedBody = None
        expectedPath = "/fake.json/fakeParam/path?includeAll=true&"
        
        self.assertRaises(wordnik.MissingParameters, self.h.process_args,path,alt_params,[],{ "parameter": "fakeParam"})
        self.assertEqual(self.h.process_args(path, alt_params, [], { "parameter": "fakeParam", "includeAll": "true" }  ),
                         (expectedPath, expectedHeaders, expectedBody) )

    def test_uncamel(self):
        self.assertEqual(self.h.uncamel("camelString"), "camel_string")
    
    def test_remove_params(self):
        self.assertEqual(self.h.remove_params("/test/{path}/with/{params}"), "/test//with/")
    
    def test_componentize(self):
        self.assertEqual(self.h.componentize("one_two_three"), ["one", "two", "three"])
        
    def test_normalize(self):
        self.assertRaises(TypeError, self.h.normalize)
        self.assertEqual(self.h.normalize(path, "get"), "fake_get_path")
    
    

if __name__ == "__main__":
    unittest.main()
  



#!/usr/bin/env python

from setuptools import setup

setup(
    name='wordnik',
    version="1.2",
    description='Simple wrapper around the wordnik API',
    author='Robin Walsh',
    author_email='robin@wordnik.com',
    test_suite = "nose.collector",
    url='http://developer.wordnik.com',
    packages = ['wordnik' ],
    package_data={'wordnik': ['endpoints/*.json']},
    setup_requires=['nose>=0.11']
)
